import React from 'react';
import ItemsMap from './ItemsMap';
import type { ItemOrElement, ScrollBehaviorArg, scrollToItemOptions, visibleElements } from './types';
type ScrollOptions = Omit<scrollToItemOptions, 'behavior'>;
export default function createApi(items: ItemsMap, visibleElementsWithSeparators?: visibleElements, transitionOptions?: {
    duration?: scrollToItemOptions['duration'];
    ease?: scrollToItemOptions['ease'];
    behavior?: scrollToItemOptions['behavior'];
    boundary?: React.MutableRefObject<scrollToItemOptions['boundary']>;
}, noPolyfill?: boolean): {
    getItemById: (id: string) => import("./types").IOItem | undefined;
    getItemElementById: (id: string | number) => Element | null;
    getItemByIndex: (index: number | string) => import("./types").IOItem | undefined;
    getItemElementByIndex: (id: string | number) => Element | null;
    getNextItem: () => import("./types").IOItem | undefined;
    getNextElement: () => import("./types").IOItem | undefined;
    getPrevItem: () => import("./types").IOItem | undefined;
    getPrevElement: () => import("./types").IOItem | undefined;
    isFirstItemVisible: boolean;
    isItemVisible: (id: string) => boolean;
    isLastItem: (id: string) => boolean;
    isLastItemVisible: boolean;
    scrollNext: (behavior?: ScrollBehaviorArg, inline?: ScrollLogicalPosition, block?: ScrollLogicalPosition, { duration, ease, boundary }?: ScrollOptions) => void;
    scrollPrev: (behavior?: ScrollBehaviorArg, inline?: ScrollLogicalPosition, block?: ScrollLogicalPosition, { duration, ease, boundary }?: ScrollOptions) => void;
    scrollToItem: (target?: ItemOrElement, behavior?: ScrollBehaviorArg, inline?: ScrollLogicalPosition, block?: ScrollLogicalPosition, options?: scrollToItemOptions) => void;
    visibleElements: visibleElements;
    visibleElementsWithSeparators: visibleElements;
    visibleItems: visibleElements;
    visibleItemsWithoutSeparators: visibleElements;
};
export interface publicApiType extends ReturnType<typeof createApi> {
    initComplete: boolean;
    items: ItemsMap;
    scrollContainer: React.RefObject<HTMLElement | null>;
    visibleElements: visibleElements;
    visibleElementsWithSeparators: visibleElements;
    /**
      Deprecated, use visibleElementsWithSeparators
     */
    visibleItems: visibleElements;
    /**
      Deprecated, use visibleElements
     */
    visibleItemsWithoutSeparators: visibleElements;
}
export {};
